use strict;
use Test::More;
use Git::Repository::Plugin::Diff;

# replace with the actual test
BEGIN {
    use_ok 'Git::Repository::Plugin::Diff';
    use_ok 'Git::Repository::Plugin::Diff::Hunk';
}

done_testing;
